pub struct Accumulator {
def_span: Span,
out_span: Option<Span>,
text: String,
errors: ErrorAccumulator,
}Expand description
Accumulator for string concatenation ${concat }
Implements ExpansionOutput and SubstParseContext:
i.e., it corresponds to the lexical context for a ${concat },
and collects the string to be output.
Fields
def_span: Spanout_span: Option<Span>text: Stringerrors: ErrorAccumulatorImplementations
Sourceimpl Accumulator
impl Accumulator
pub fn finish_literal(self) -> Result<LitStr>
pub fn finish_onto<O>(self, np: &O::NotInPaste, out: &mut O)where
O: ExpansionOutput,
fn append_display(&mut self, span: Span, plain: impl Display + Spanned)
Sourcefn append_type_like(&mut self, tokens: TokenStream)
fn append_type_like(&mut self, tokens: TokenStream)
Convert a type to a “nice” string
syn::Type as ToTokens produces a lot of Spacing::Alone
which results in a lot of excessive whitespace.
We use an algorithm which prints something reasonably pretty
which we also think is probably parseable
unless there None-delimited groups (which we make visible).
Note that our documentation doesn’t promise parseability, just readability.
This algorithm does obviously produce parseable output if the input is just a single identifier token.
Rules
- We never put spaces next to any of
<>: - We don’t put a space after
&or' - We don’t put a space before
, - We put spaces just inside
{ }iff nonempty - We don’t put spaces just inside
( )or[ ](or« ») - Otherwise, we put a space between each pair of TTs